﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO.Ports;
using System.Threading;

namespace ir_kit
{
    public partial class Form1 : Form
    {
        SerialPort ir_port = null;
        IRShield irs = null;
        Thread ir_th = null;
        bool ir_th_loop;
        IRFalcon ir_falcon = null;

        BrainBandParser bb = null;
        Thread bb_th = null;
        bool bb_th_loop;
        Point[] bb_raw;

        delegate void IRPacketDelegate(IRPacket p);
        delegate void ThrottleDelegate();
        delegate void ErrorMessageDelegate(string s);

        public Form1()
        {
            InitializeComponent();
        }

        public void ir_open(string p)
        {
            ir_port = new SerialPort(p, 19200, Parity.None, 8, StopBits.One);
            if (ir_port != null)
            {
                ir_port.Open();
                if (ir_port.IsOpen)
                {
                    irs = new IRShield(ir_port);
                    ir_falcon = new IRFalcon(ir_port);
                }
                else
                {
                    throw new Exception("IR port can't open.");
                }
            }
            else
            {
                throw new Exception("IR port can't create.");
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            foreach (string p in SerialPort.GetPortNames())
            {
                comboBox1.Items.Add(p);
                comboBox2.Items.Add(p);
            }
            if (comboBox1.Items.Count > 0)
                comboBox1.Text = comboBox1.Items[0].ToString();
            if (comboBox2.Items.Count > 0)
                comboBox2.Text = comboBox2.Items[0].ToString();

            toolStripStatusLabel1.ForeColor = Color.Red;
            toolStripStatusLabel1.Text = "";
        }

        private void stop_thread(Thread t)
        {
            if (t != null && t.IsAlive)
            {
                while (t.IsAlive)
                {
                    t.Abort();
                    Thread.Sleep(10);
                }
            }
        }

        /// <summary>
        /// Open an IR port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                if (ir_port == null)
                {
                    ir_open(comboBox1.Items[comboBox1.SelectedIndex].ToString());

                    ir_th = new Thread(new ThreadStart(this.ir_receiving_thread));
                    ir_th.IsBackground = true;
                    ir_th.Start();

                    button1.Text = "Close";
                    comboBox1.Enabled = false;
                }
                else
                {
                    ir_th_loop = false; Thread.Sleep(100);
                    
                    stop_thread(ir_th);
                    if (ir_port != null && ir_port.IsOpen) ir_port.Close();

                    ir_port = null;
                    ir_th = null;
                    irs = null;
                    ir_falcon = null;

                    button1.Text = "Open";
                    comboBox1.Enabled = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// Open a brain band port.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                if (bb == null)
                {
                    bb = new BrainBandParser(comboBox2.Items[comboBox2.SelectedIndex].ToString());

                    bb_raw = new Point[bb.RawData.Length];
                    bb_th = new Thread(new ThreadStart(this.bb_receiving_thread));
                    bb_th.IsBackground = true;
                    bb_th.Start();

                    button2.Text = "Close";
                    comboBox2.Enabled = false;
                }
                else
                {
                    bb_th_loop = false; Thread.Sleep(100);

                    stop_thread(bb_th);
                    bb.close();
                    bb = null;
                    bb_th = null;
                    bb_raw = null;

                    button2.Text = "Open";
                    comboBox2.Enabled = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        internal void add_packet(IRPacket p)
        {
            listBox1.Items.Insert(0, p);

            if (listBox1.Items.Count > 100)
                listBox1.Items.RemoveAt(listBox1.Items.Count - 1);
        }

        internal void set_attention_to_throttle()
        {
            if (bb.SignalQuality < 30)
            {
                trackBar1.Value = (bb.Attention - 20) * 63 / 80;
            }
            else
            {
                trackBar1.Value = 0;
            }
        }

        internal void add_error_message(string s)
        {
            toolStripStatusLabel1.Text = DateTime.Now.ToLongTimeString() + " " + s;
        }

        private void ir_receiving_thread()
        {
            ir_th_loop = true;
            while (ir_th_loop)
            {
                try
                {
                    IRPacket p = irs.receive();
                    if (p != null)
                    {
                        Invoke(new IRPacketDelegate(add_packet), p);
                    }
                }
                catch (Exception ex)
                {
                    Invoke(new ErrorMessageDelegate(add_error_message), ex.Message);
                }
                Thread.Sleep(10);
            }
        }

        private void bb_receiving_thread()
        {
            int pidx = -1;
            bb_th_loop = true;
            while (bb_th_loop)
            {
                try
                {
                    if (bb.update())
                    {
                        if (bb.LastPacket != 0x80)
                        {
                            Console.WriteLine(DateTime.Now.ToLongTimeString());
                            Console.WriteLine(bb.ToString());
                            Console.WriteLine();

                            Invoke(new ThrottleDelegate(set_attention_to_throttle));
                        }
                        else
                        {
                            if (bb.RawIndex != pidx
                                && bb.RawIndex % 5 == 0)
                            {
                                int[] raw = bb.RawData;
                                for (int i = 0; i < bb.RawData.Length; i++)
                                {
                                    bb_raw[i].X = i;
                                    bb_raw[i].Y = pictureBox1.Height / 2 + (raw[i] * (pictureBox1.Height / 2) / 4096);
                                }
                                pictureBox1.Invalidate();
                                pidx = bb.RawIndex;
                            }
                        }
                    }
                }
                catch (TimeoutException ex)
                {
                    Invoke(new ErrorMessageDelegate(add_error_message), ex.Message);
                }
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            ir_th_loop = bb_th_loop = false; Thread.Sleep(100); 
            stop_thread(ir_th);
            stop_thread(bb_th);

            if (ir_port != null && ir_port.IsOpen) ir_port.Close();
        }

        /// <summary>
        /// Send IR packet for the Falcon.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            if (ir_falcon != null)
            {
                ir_falcon.send((byte)0, (byte)trackBar1.Value, (char)0, (char)0, (char)0);
            }
        }

        private void checkBox1_CheckStateChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked)
                timer1.Start();
            else
                timer1.Stop();
        }

        private void send_ir_packet()
        {
            if (irs == null)
            {
                add_error_message("COM port has not been opened yet.");
                return;
            }
            if (listBox1.SelectedItems.Count > 0)
            {
                IRPacket[] ps = get_selected_packets();
                ir_port.ReadTimeout = 1000;
                foreach (IRPacket p in ps)
                {
                    if (irs.send(p, true) < 0)
                    {
                        add_error_message("Data send error.");
                    }
                }
            }
            else
            {
                add_error_message("Did not select an IR Packet in the received list.");
            }
        }

        /// <summary>
        /// Send IR packet.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listBox1_DoubleClick(object sender, EventArgs e)
        {
            send_ir_packet();
        }

        /// <summary>
        /// Send IR packet.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button3_Click(object sender, EventArgs e)
        {
            send_ir_packet();
        }

        /// <summary>
        /// Draw a RAW data.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            if (bb_raw == null) return;
            e.Graphics.DrawLines(Pens.Black, bb_raw);
        }

        /// <summary>
        /// Save IR Packet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void saveIRPacketToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedItems.Count > 0)
            {
                SaveFileDialog sfd = new SaveFileDialog();
                sfd.FileName = "ir_packet.txt";
                sfd.Filter = "IR Packet(*.txt)|*.txt|All files(*.*)|*.*";
                sfd.RestoreDirectory = true;

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    IRPacket.save(sfd.FileName,get_selected_packets());
                }
            }
            else
            {
                MessageBox.Show("Did not select an IR Packet in the received list.");
            }
        }

        /// <summary>
        /// Load IR Packet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void openIRPacketOToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "IR Packet(*.txt)|*.txt|All files(*.*)|*.*";
            ofd.RestoreDirectory = true;

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                IRPacket[] ps = IRPacket.load(ofd.FileName);
                if (ps != null)
                {
                    foreach(IRPacket p in ps) add_packet(p);
                }
            }
        }

        private IRPacket[] get_selected_packets()
        {
            List<IRPacket> items = new List<IRPacket>();
            foreach (IRPacket o in listBox1.SelectedItems)
            {
                items.Add(o);
            }
            items.Reverse();
            return items.ToArray();
        }

        /// <summary>
        /// Delete selected items in the listbox1.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listBox1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                IRPacket[] ps = get_selected_packets();
                foreach (IRPacket p in ps) listBox1.Items.Remove(p);
                /*
                List<Object> items = new List<object>();
                foreach (Object o in listBox1.SelectedItems)
                {
                    items.Add(o);
                }
                foreach (Object o in items) listBox1.Items.Remove(o);
                 */
            }
        }

    }
}
