﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;

namespace ir_kit
{
    public class IRFalcon : IRShield
    {
        public IRFalcon(SerialPort p) :
            base(p)
        {
            pkt.header_h = 90;  // 2360 usec / 26.32 usec
            pkt.header_l = 27;  // 700 usec / 26.32 usec
            pkt.data1_h = 61;   // 1600 usec / 26.32 usec
            pkt.data1_l = 27;   // 700 usec / 26.32 usec
            pkt.data0_h = 33;   // 860 usec / 26.32 usec
            pkt.data0_l = 27;   // 700 usec / 26.32 usec
            pkt.trailer_h = 33; // 860 usec / 26.32 usec
            pkt.trailer_l = 38; // 1000 usec / 26.32 usec
            pkt.alloc_data(23); // 23 bits
        }

        // short format
        // ch A=0,B=2,C=1
        // ud 0 - 63
        // fb -15 - 15
        // lr -15 - 15
        // tr -15 - 15
        public void send(byte ch, byte ud, char fb, char lr, char tr)
        {
            if (ud > 0)
            {
                pkt.data[0] = (byte)(ch << 6 | (ud & 0x3f));
                pkt.data[1] = (byte)(Math.Abs(fb) << 4 | (Math.Abs(lr) & 0x0f));
                pkt.data[2] = (byte)((Math.Abs(tr) & 0x0f) << 1);
                if (fb >= 0) pkt.data[2] |= 0x80;
                if (lr > 0) pkt.data[2] |= 0x40;
                if (tr > 0) pkt.data[2] |= 0x20;
            }
            else
            {
                pkt.data[0] = pkt.data[1] = pkt.data[2] = 0;
            }
            send(pkt);
        }
    }

    public class IRMiniBee : IRShield
    {
        public IRMiniBee(SerialPort p) :
            base(p)
        {
            pkt.header_h = 105;  // 2760 usec / 26.32 usec
            pkt.header_l = 95;   // 2500 usec / 26.32 usec
            pkt.data1_h = 46;    // 1200 usec / 26.32 usec
            pkt.data1_l = 32;    // 840 usec / 26.32 usec
            pkt.data0_h = 30;    // 800 usec / 26.32 usec
            pkt.data0_l = 18;    // 470 usec / 26.32 usec
            pkt.trailer_h = 23;  // 600 usec / 26.32 usec
            pkt.trailer_l = 38;  // 1000 usec / 26.32 usec
            pkt.alloc_data(18);  // 18 bits
        }

        // lr 0 - 7
        // ud 0 - 31
        // tr L = 2, R = 1
        // ch A = 3, B = 2, C = 1
        public void send(byte ch, byte ud, byte lr, byte tr)
        {
            if (ud > 0)
            {
                byte cs = (byte)((lr + ud + tr + ch) & 0xf);
                pkt.data[0] = (byte)((lr << 5) | (ud & 0x1f));
                pkt.data[1] = (byte)((tr << 6) | (ch & 0x03) << 4 | cs);
                pkt.data[2] = 0;
            }
            else
            {
                pkt.data[0] = pkt.data[1] = pkt.data[2] = 0;
            }
            send(pkt);
        }
    }
}
